VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "cWriteEventLog"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' ************************************************************
' Copyright  1996-2001 Slightly Tilted Software
' All rights reserved
' You're absolutely free to use these resources within your
'     own applications, but you may not redistribute them
'     (as source) in any manner whatsoever, whether for profit
'     or not.
' The only legitimate source for the original source code is
'     at the VBPJ site and my own web site at:
'     http://www.SlightlyTiltedSoftware.com
' ************************************************************

' ---------------------------------------------
' Module    : WriteEvents.cls
' By        : L.J. Johnson       Date: 04-28-2001
' Comments  : Class to write an item to the NT
'           : event log.
' ---------------------------------------------
Option Explicit

' --------------------------------------------
' Public enum for the error levels and log types
' --------------------------------------------
Public Enum enmErrLevel
   lInfo = &H60000000
   lWarning = &HA0000000
   lError = &HE0000000
End Enum

Public Enum enmLogType
   LogError = 1&
   LogWarning = 2&
   LogInfo = 4&
   AuditSuccess = 8&
   AuditFailure = 16&
End Enum

' ---------------------------------------------
' Used to return errors from RegisterEvntSource
' ---------------------------------------------
Private m_lngErrNum                    As Long
Private m_strErrSource                 As String
Private m_strErrDesc                   As String

' ---------------------------------------------
' These are the available Event log types.
' ---------------------------------------------
Private Const EVENTLOG_ERROR_TYPE       As Long = &H1
Private Const EVENTLOG_WARNING_TYPE     As Long = &H2
Private Const EVENTLOG_INFORMATION_TYPE As Long = &H4
Private Const EVENTLOG_AUDIT_SUCCESS    As Long = &H8
Private Const EVENTLOG_AUDIT_FAILURE    As Long = &H10

' ---------------------------------------------
' Module-level variables
' ---------------------------------------------
Private m_lngEvtLogHandle              As Long
Private m_lngCategory                  As Long
Private m_lngNumMsgs                   As Long
Private m_lngLenRawData                As Long
Private m_typMessages                  As FmtMsgArrayType
Private m_abytDataBuffer()             As Byte

' ---------------------------------------------
' API declarations for Event logs
' ---------------------------------------------
Private Declare Function OpenEventLog _
   Lib "advapi32" Alias "OpenEventLogA" _
   (ByVal lpUNCServerName As String, _
    ByVal lpSourceName As String) As Long
Private Declare Function CloseEventLog _
   Lib "advapi32" _
   (ByVal hEventLog As Long) As Long
Private Declare Function RegisterEventSource _
   Lib "advapi32" Alias "RegisterEventSourceA" _
   (ByVal lpUNCServerName As String, _
    ByVal lpSourceName As String) As Long
Private Declare Function DeregisterEventSource _
   Lib "advapi32" _
   (ByVal hEventLog As Long) As Long
Private Declare Function ReportEvent _
   Lib "advapi32" Alias "ReportEventA" _
   (ByVal hEventLog As Long, _
    ByVal wType As Long, _
    ByVal wCategory As Long, _
    ByVal dwEventID As Long, _
    ByVal lpUserSid As Long, _
    ByVal wNumStrings As Long, _
    ByVal dwDataSize As Long, _
    lpStrings As Any, _
    lpRawData As Any) As Long

' ---------------------------------------------
' Used for info on SIDs
' ---------------------------------------------
Private Enum TOKEN_INFORMATION_CLASS
    TokenUser = 1&
    TokenGroups = 2&
    TokenPrivileges = 3&
    TokenOwner = 4&
    TokenPrimaryGroup = 5&
    TokenDefaultDacl = 6&
    TokenSource = 7&
    TokenType = 8&
    TokenImpersonationLevel = 9&
    TokenStatistics = 10&
    TokenRestrictedSids = 11&
    TokenSessionId = 12&
End Enum

Private Type SID_AND_ATTRIBUTES
    pSid                               As Long
    hAttributes                        As Long
End Type

Private Type TOKEN_USER
    uSid                               As SID_AND_ATTRIBUTES
End Type

Private Const TOKEN_QUERY              As Long = &H8&

Private Declare Function CloseHandle _
    Lib "kernel32" ( _
    ByVal hObject As Long) As Long
Private Declare Function OpenProcessToken _
    Lib "advapi32" ( _
    ByVal hProcess As Long, _
    ByVal dwDesiredAccess As Long, _
    hToken As Long) As Long
Private Declare Function GetCurrentProcess _
    Lib "kernel32" () As Long
Private Declare Function GetTokenInformation _
    Lib "advapi32.dll" ( _
    ByVal hToken As Long, _
    ByVal eTokenInformationClass As TOKEN_INFORMATION_CLASS, _
    uTokenInformation As Any, _
    ByVal nTokenInformationLength As Long, _
    nReturnLength As Long) As Long

' ---------------------------------------------
' Used as part of error string return
' ---------------------------------------------
Private Const REG_SOURCENAME           As String = "WriteEvents.WriteEventsLog."

' *******************************************************
' Routine Name : (PUBLIC in CLASS) Function LogNames
' Written By   : L.J. Johnson
' Programmer   : L.J. Johnson [Slightly Tilted Software]
' Date Writen  : 04/28/2001 -- 13:30:52
' Inputs       : ByVal xi_enmOSType:enmValidOSs -
' Outputs      : Variant --
' Description  : Get the names of the event logs --
'              :     Standard 3 in NT, more in
'              :     W2K and Whistler
' Called By    : *****
' *******************************************************
Public Function LogNames(ByVal xi_enmOSType As enmValidOSs) As Variant
   Dim p_vntNames()                    As Variant
   Dim p_objRegEventSource             As cRegisterEventSource
   
   If xi_enmOSType = Valid_NT Or _
          xi_enmOSType = Valid_Win2000 Or _
          xi_enmOSType = Valid_Whistler Then
      Set p_objRegEventSource = New cRegisterEventSource
      p_vntNames = p_objRegEventSource.GetLogNames()
   End If
   
   LogNames = p_vntNames
   
End Function

' *******************************************************
' Routine Name : (PUBLIC in CLASS) Sub WriteEventLog
' Written By   : L.J. Johnson
' Programmer   : L.J. Johnson [Slightly Tilted Software]
' Date Writen  : 04/28/2001 -- 14:24:35
' Inputs       : ByVal xi_strServerName:String     - Fully UNC-qualified name of the server ("" for local server)
'              : ByVal xi_strAppName:String        - subkey of a logfile entry under the EventLog key in the registry
'              : ByVal xi_enmLogType:xi_enmLogType - Error, Warning, Information, Audit Success, or Audit Failure
'              : byval xi_lngEventID:Long          - EventID from the message resource file
'              : ByRef xio_aEventText():String     - Replaceable parameters for log message
' Outputs      : N/A
' Description  : Write an item to the Event log
' Called By    : *****
' *******************************************************
Public Sub WriteEventLog(ByVal xi_strServerName As String, _
                         ByVal xi_strAppName As String, _
                         ByVal xi_enmLogType As enmLogType, _
                         ByVal xi_lngEventID As Long, _
                         ByVal xi_strEventLogName As String, _
                         ByRef xio_aEventText() As String)
   Dim p_lngRtn                        As Long
   Dim p_lngRegEvntLogHwnd             As Long
   Dim p_lngUserSid                    As Long
   Dim p_lngNumMessages                As Long
   Dim p_strReplace                    As String
   Dim p_strServerName                 As String
   
   ' ------------------------------------------
   ' Add the leading '\\' to remote server names
   ' ------------------------------------------
   If Trim$(xi_strServerName) = vbNullString Then
      p_strServerName = vbNullString
   Else
      If Left$(Trim$(xi_strServerName), 2) <> "\\" Then
         p_strServerName = "\\" & xi_strServerName
      Else
         p_strServerName = xi_strServerName
      End If
   End If

   ' ------------------------------------------
   ' Get the number of messages
   ' ------------------------------------------
   p_lngNumMessages = CLng(TranslateArray(xio_aEventText(), m_typMessages))

   ' ------------------------------------------
   ' Register the event log
   ' ------------------------------------------
   p_lngRegEvntLogHwnd = RegisterEventSource(lpUNCServerName:=p_strServerName, _
                                             lpSourceName:=xi_strAppName)

   If p_lngRegEvntLogHwnd = 0 Then
      p_lngRtn = Err.LastDllError
      
      m_lngErrNum = (ERR_REG_EVENT_SOURCE + vbObjectError)
      m_strErrSource = REG_SOURCENAME & "WriteEventLog"
      m_strErrDesc = "Could not RegisterEventSource for " & _
                    xi_strAppName & ".  The error was: " & _
                    ReturnApiErrString(p_lngRtn)
      Err.Raise Number:=m_lngErrNum, _
                Description:=m_strErrDesc, _
                Source:=m_strErrSource
   End If

   ' ------------------------------------------
   ' Get the user of the current process's SID
   '     and use that to insert this user's name
   '     in the log item
   ' ------------------------------------------
   p_lngUserSid = GetSid()
   
   ' ------------------------------------------
   ' Write to the application event log
   ' ------------------------------------------
   p_lngRtn = ReportEvent(hEventLog:=p_lngRegEvntLogHwnd, _
                          wType:=xi_enmLogType, _
                          wCategory:=m_lngCategory, _
                          dwEventID:=xi_lngEventID, _
                          lpUserSid:=p_lngUserSid, _
                          wNumStrings:=p_lngNumMessages, _
                          dwDataSize:=m_lngLenRawData, _
                          lpStrings:=m_typMessages, _
                          lpRawData:=m_abytDataBuffer(0))

   ' ------------------------------------------
   ' Check for errors
   ' ------------------------------------------
   If p_lngRtn = False Then
      p_lngRtn = Err.LastDllError
      
      m_lngErrNum = (ERR_REPORT_EVENT + vbObjectError)
      m_strErrSource = REG_SOURCENAME & "WriteEventLog"
      m_strErrDesc = "Could not WRITE to the Application Event Log. " & _
                              "The error was: " & ReturnApiErrString(p_lngRtn)
      Err.Raise Number:=m_lngErrNum, _
                Description:=m_strErrDesc, _
                Source:=m_strErrSource
   End If

   ' ------------------------------------------
   ' Cleanup -- DeRegister the event log
   ' ------------------------------------------
   p_lngRtn = DeregisterEventSource(p_lngRegEvntLogHwnd)

   If p_lngRtn = False Then
      p_lngRtn = Err.LastDllError
      
      m_lngErrNum = (ERR_DEREG_EVENT_SOURCE + vbObjectError)
      m_strErrSource = REG_SOURCENAME & "WriteEventLog"
      m_strErrDesc = "Could not WRITE to the Application Event Log." & _
                              "The error was: " & ReturnApiErrString(p_lngRtn)
      Err.Raise Number:=m_lngErrNum, _
                Description:=m_strErrDesc, _
                Source:=m_strErrSource
   End If

End Sub

' *******************************************************
' Routine Name : (PUBLIC in CLASS) Property Get EventDataBuffer
' Written By   : L.J. Johnson
' Programmer   : L.J. Johnson [Slightly Tilted Software]
' Date Writen  : 04/28/2001 -- 14:20:20
' Inputs       : N/A
' Outputs      : String --
' Description  : Get/Let the Data Buffer (binary)
' Called By    :
' *******************************************************
Public Property Get EventDataBuffer() As Variant
   EventDataBuffer = m_abytDataBuffer
End Property

Public Property Let EventDataBuffer(ByRef xi_abytData As Variant)
   m_abytDataBuffer = xi_abytData
   m_lngLenRawData = UBound(xi_abytData)
End Property

' *******************************************************
' Routine Name : (PUBLIC in CLASS) Property Get EventCategory
' Written By   : L.J. Johnson
' Programmer   : L.J. Johnson [Slightly Tilted Software]
' Date Writen  : 04/28/2001 -- 14:19:42
' Inputs       : N/A
' Outputs      : Long --
' Description  : Get/Let the Event Category
' Called By    : *****
' *******************************************************
Public Property Get EventCategory() As Long
   EventCategory = m_lngCategory
End Property

Public Property Let EventCategory(ByVal xi_lngEventCategory As Long)
   m_lngCategory = xi_lngEventCategory
End Property

' *******************************************************
' Routine Name : (PRIVATE in CLASS) Function GetSid
' Written By   : L.J. Johnson
' Programmer   : L.J. Johnson [Slightly Tilted Software]
' Date Writen  : 04/28/2001 -- 14:17:39
' Inputs       : N/A
' Outputs      : Long --
' Description  : Get the user's SID
' Called By    : WriteEventLog
' *******************************************************
Private Function GetSid() As Long
   Const FIRST_ATTEMPT                 As Long = 16&
   Dim p_lngToken                      As Long
   Dim p_atypTokenInfo()               As TOKEN_USER
   Dim p_lngLength                     As Long
   Dim p_lngRtn                        As Long
   
   If OpenProcessToken(GetCurrentProcess, _
                       TOKEN_QUERY, _
                       p_lngToken) Then
      If p_lngToken <> 0 Then
         ReDim p_atypTokenInfo(1 To FIRST_ATTEMPT) As TOKEN_USER
         p_lngRtn = GetTokenInformation(hToken:=p_lngToken, _
                                        eTokenInformationClass:=TokenUser, _
                                        uTokenInformation:=p_atypTokenInfo(1), _
                                        nTokenInformationLength:=Len(p_atypTokenInfo(1)) * FIRST_ATTEMPT, _
                                        nReturnLength:=p_lngLength)
         If p_lngRtn <> 0 Then
            If p_atypTokenInfo(1).uSid.pSid <> 0 Then
               GetSid = p_atypTokenInfo(1).uSid.pSid
            Else
               GetSid = 0&
            End If
         Else
            GetSid = 0&
         End If
      Else
         GetSid = 0&
      End If
   Else
      GetSid = 0&
   End If
   
   If p_lngToken <> 0 Then
      CloseHandle p_lngToken
   End If
End Function
